PAISES = {
    'AD': 'Principado de Andorra',
    'AE': 'Emiratos Arabes Unidos',
    'AF': 'Afganistán',
    'AG': 'Antigua y Bermuda',
    'AI': 'Isla Anguilla',
    'AL': 'República de Albania',
    'AN': 'Antillas Neerlandesas',
    'AO': 'República de Angola',
    'AQ': 'Antártica',
    'AR': 'Argentina',
    'AS': 'Samoa Americana',
    'AT': 'Austria',
    'AU': 'Australia',
    'AW': 'Aruba',
    'AX': 'Ascensión',
    'AZ': 'Islas Azores',
    'BB': 'Barbados',
    'BD': 'Bangladesh',
    'BE': 'Bélgica',
    'BF': 'Burkina Faso',
    'BG': 'Bulgaria',
    'BH': 'Estado de Bahrein',
    'BI': 'Burundi',
    'BJ': 'Benin',
    'BL': 'Belice',
    'BM': 'Bermudas',
    'BN': 'Brunei Darussalam',
    'BO': 'Bolivia',
    'BR': 'Brasil',
    'BS': 'Commonwealth de las Bahamas',
    'BT': 'Buthan',
    'BU': 'Burma',
    'BV': 'Isla Bouvet',
    'BW': 'Botswana',
    'BY': 'Bielorrusia',
    'CA': 'Canadá',
    'CC': 'Isla de Cocos o Kelling',
    'CD': 'Islas Canarias',
    'CE': 'Isla de Christmas',
    'CF': 'República Centro Africana',
    'CG': 'Congo',
    'CH': 'Suiza',
    'CI': 'Costa de Marfil',
    'CK': 'Islas Cook',
    'CL': 'Chile',
    'CM': 'Camerún',
    'CN': 'China',
    'CO': 'Colombia',
    'CP': "Campione D'Italia",
    'CR': 'República de Costa Rica',
    'CS': 'República Checa y República Eslovaca',
    'CU': 'Cuba',
    'CV': 'República de Cabo Verde',
    'CX': 'Isla de Navidad',
    'CY': 'República de Chipre',
    'DD': 'Alemania',
    'DJ': 'República de Djibouti',
    'DK': 'Dinamarca',
    'DM': 'República Dominicana',
    'DN': 'Commonwealth de Dominica',
    'DZ': 'Argelia',
    'EC': 'Ecuador',
    'EG': 'Egipto',
    'EH': 'Sahara del Oeste',
    'EO': 'Estado Independiente de Samoa Occidental',
    'ES': 'España',
    'ET': 'Etiopía',
    'FI': 'Finlandia',
    'FJ': 'Fiji',
    'FK': 'Islas Malvinas',
    'FM': 'Micronesia',
    'FO': 'Islas Faroe',
    'FR': 'Francia',
    'GA': 'Gabón',
    'GB': 'Gran Bretaña (Reino Unido)',
    'GD': 'Granada',
    'GF': 'Guyana Francesa',
    'GH': 'Ghana',
    'GI': 'Gibraltar',
    'GJ': 'Groenlandia',
    'GM': 'Gambia',
    'GN': 'Guinea',
    'GP': 'Guadalupe',
    'GQ': 'Guinea Ecuatorial',
    'GR': 'Grecia',
    'GT': 'Guatemala',
    'GU': 'Guam',
    'GW': 'Guinea Bissau',
    'GY': 'República de Guyana',
    'GZ': 'Islas de Guernesey, Jersey, Alderney, Isla Great Sark, Herm, Little Sark, Berchou, Jethou, Lihou (Islas del Canal)',
    'HK': 'Hong Kong',
    'HM': 'Islas Heard and Mc Donald',
    'HN': 'República de Honduras',
    'HT': 'Haití',
    'HU': 'Hungría',
    'ID': 'Indonesia',
    'IE': 'Irlanda',
    'IH': 'Isla del Hombre',
    'IL': 'Israel',
    'IN': 'India',
    'IO': 'Territorio Británico en el Océano Indico',
    'IP': 'Islas Pacífico',
    'IQ': 'Iraq',
    'IR': 'Irán',
    'IS': 'Islandia',
    'IT': 'Italia',
    'JM': 'Jamaica',
    'JO': 'Reino Hachemita de Jordania',
    'JP': 'Japón',
    'KE': 'Kenia',
    'KH': 'Campuchea Democrática',
    'KI': 'Kiribati',
    'KM': 'Comoros',
    'KN': 'San Kitts',
    'KP': 'República Democrática de Corea',
    'KR': 'República de Corea',
    'KW': 'Estado de Kuwait',
    'KY': 'Islas Caimán',
    'LA': 'República Democrática de Laos',
    'LB': 'Líbano',
    'LC': 'Santa Lucía',
    'LI': 'Principado de Liechtenstein',
    'LK': 'República Socialista Democrática de Sri Lanka',
    'LN': 'Labuán',
    'LR': 'República de Liberia',
    'LS': 'Lesotho',
    'LU': 'Gran Ducado de Luxemburgo',
    'LY': 'Libia',
    'MA': 'Marruecos',
    'MC': 'Principado de Mónaco',
    'MD': 'Madeira',
    'MG': 'Madagascar',
    'MH': 'República de las Islas Marshall',
    'ML': 'Malí',
    'MN': 'Mongolia',
    'MO': 'Macao',
    'MP': 'Islas Marianas del Noreste',
    'MQ': 'Martinica',
    'MR': 'Mauritania',
    'MS': 'Monserrat',
    'MT': 'Malta',
    'MU': 'República de Mauricio',
    'MV': 'República de Maldivas',
    'MW': 'Malawi',
    'MY': 'Malasia',
    'MZ': 'Mozambique',
    'NA': 'República de Namibia',
    'NC': 'Nueva Caledonia',
    'NE': 'Níger',
    'NF': 'Isla de Norfolk',
    'NG': 'Nigeria',
    'NI': 'Nicaragua',
    'NL': 'Holanda',
    'NO': 'Noruega',
    'NP': 'Nepal',
    'NR': 'República de Nauru',
    'NT': 'Zona Neutral',
    'NU': 'Niue',
    'NV': 'Nevis',
    'NZ': 'Nueva Zelandia',
    'OM': 'Sultanía de Omán',
    'PA': 'República de Panamá',
    'PE': 'Perú',
    'PF': 'Polinesia Francesa',
    'PG': 'Papúa Nueva Guinea',
    'PH': 'Filipinas',
    'PK': 'Pakistán',
    'PL': 'Polonia',
    'PM': 'Isla de San Pedro y Miguelón',
    'PN': 'Pitcairn',
    'PR': 'Estado Libre Asociado de Puerto Rico',
    'PT': 'Portugal',
    'PU': 'Patau',
    'PW': 'Palau',
    'PY': 'Paraguay',
    'QA': 'Estado de Quatar',
    'QB': 'Isla Qeshm',
    'RE': 'Reunión',
    'RO': 'Rumania',
    'RW': 'Rhuanda',
    'SA': 'Arabia Saudita',
    'SB': 'Islas Salomón',
    'SC': 'Seychelles Islas',
    'SD': 'Sudán',
    'SE': 'Suecia',
    'SG': 'Singapur',
    'SH': 'Santa Elena',
    'SI': 'Archipiélago de Svalbard',
    'SJ': 'Islas Svalbard and Jan Mayen',
    'SK': 'Sark',
    'SL': 'Sierra Leona',
    'SM': 'Serenísima República de San Marino',
    'SN': 'Senegal',
    'SO': 'Somalia',
    'SR': 'Surinam',
    'ST': 'Sao Tome and Príncipe',
    'SU': 'Países de la Ex-U.R.S.S., excepto Ucrania y Bielorusia',
    'SV': 'El Salvador',
    'SW': 'República de Seychelles',
    'SY': 'Siria',
    'SZ': 'Reino de Swazilandia',
    'TC': 'Islas Turcas y Caicos',
    'TD': 'Chad',
    'TF': 'Territorios Franceses del Sureste',
    'TG': 'Togo',
    'TH': 'Thailandia',
    'TK': 'Tokelau',
    'TN': 'República de Túnez',
    'TO': 'Reino de Tonga',
    'TP': 'Timor Este',
    'TR': 'Trieste',
    'TS': 'Tristán Da Cunha',
    'TT': 'República de Trinidad y Tobago',
    'TU': 'Turquía',
    'TV': 'Tuvalu',
    'TW': 'Taiwán',
    'TZ': 'Tanzania',
    'UA': 'Ucrania',
    'UG': 'Uganda',
    'UM': 'Islas menores alejadas de los Estados Unidos',
    'US': 'Estados Unidos de América',
    'UY': 'República Oriental del Uruguay',
    'VA': 'El Vaticano',
    'VC': 'San Vicente y Las Granadinas',
    'VE': 'Venezuela',
    'VG': 'Islas Vírgenes Británicas',
    'VI': 'Islas Vírgenes de Estados Unidos de América',
    'VN': 'Vietnam',
    'VU': 'República de Vanuatu',
    'WF': 'Islas Wallis y Funtuna',
    'XX': 'Otro',
    'YD': 'Yemen Democrática',
    'YE': 'República del Yemen',
    'YU': 'Países de la Ex-Yugoslavia',
    'ZA': 'Sudáfrica',
    'ZC': 'Zona Especial Canaria',
    'ZM': 'Zambia',
    'ZO': 'Zona Libre Ostrava',
    'ZR': 'Zaire',
    'ZW': 'Zimbawe',
}
PERIODIC = {
    '0': 'Sin Seleccion',
    '1': 'Mensual',
    '2': 'Bimestral',
    '3': 'Trimestral',
    '4': 'Cuatrimestral',
    '5': 'Semestral',
}
PERIODO = {
    "00": "Sin Seleccion",
    "01": "Enero",
    "02": "Febrero",
    "03": "Marzo",
    "04": "Abril",
    "05": "Mayo",
    "06": "Junio",
    "07": "Julio",
    "08": "Agosto",
    "09": "Septiembre",
    "10": "Octubre",
    "11": "Noviembre",
    "12": "Diciembre",
    "13": "Enero - Febrero",
    "14": "Marzo - Abril",
    "15": "Mayo - Junio",
    "16": "Julio - Agosto",
    "17": "Septiembre - Octubre",
    "18": "Noviembre - Diciembre",
    "19": "Enero - Marzo",
    "20": "Abril - Junio",
    "21": "Julio - Septiembre",
    "22": "Octubre - Diciembre",
    "23": "Enero - Abril",
    "24": "Mayo - Agosto",
    "25": "Septiembre - Diciembre",
    "26": "Enero - Junio",
    "27": "Julio - Diciembre",
}
PRESENTA = {
    '0': 'La presenta sin operaciones',
    '1': 'La presenta con datos',
}
TIP_DECLA = {
    '1': 'Normal',
    '2': 'Complementaria',
}
TIP_OPERA = {
    '03': 'Prestación de Servicios Profesionales',
    '06': 'Arrendamiento de Inmuebles',
    '85': 'Otros',
}
TIPO_TERC = {
    '04': 'Proveedor Nacional',
    '05': 'Proveedor Extranjero',
    '15': 'Proveedor Global',
}
