@page {
	{% if c.Version %}
	@bottom-left{
		content: {{ k | tojson }} " versión: " {{ c.Version | tojson }};
		font:7.47pt Arial;
	}
	{% endif %}
	@bottom-center{
		content: "Este documento es una representación impresa de un CFDI";
		font:7.47pt Arial;
	}
	@bottom-right{
		content: "Página " counter(page) " de " counter(pages);
		font:7.47pt Arial;
	}
}
@media print {
    .pagebreak { page-break-before: always; } /* page-break-after works, as well */
}
body {
	font-family: Arial;
	font-size: 9pt;
}
h3 {
	margin-top:-10px;
	margin-bottom:0px;
	font-size: 9.89pt;
	background-color:#333;
	padding-top: 3px;
	padding-bottom: 3px;
	text-align: center;
	color: white;
	letter-spacing: 4px;
	font-weight: bold;
}
h4 {
	margin-top:0px;
	margin-bottom:0px;
	font-size: 9.89pt;
	background-color:#555;
	padding-top: 3px;
	padding-bottom: 3px;
	text-align: center;
	color: white;
	letter-spacing: 4px;
	font-weight: bold;
}
h5 {
	margin-top:0px;
	margin-bottom:0px;
	font-size: 8.6pt;
	background-color:#777;
	padding-top: 2.5px;
	padding-bottom: 2.5px;
	padding-left: 10px;
	text-align: left;
	color: white;
	letter-spacing: 2px;
	font-weight: bold;
}
.h {
	font-size: 9.89pt;
	padding-top: 2px;
	padding-bottom: 2px;
}
.c {
	font-size: 8.6pt;
}
.ci {
	font-size: 7.47pt;
}
.cii {
	font-size: 6.5pt;
}
.ciii {
	font-size: 5.65pt;
}
table {
	font-size: 8.6pt;
	width: 100%;
	border-collapse: collapse;
	border: none;
	border-spacing: 0;
}
table table {
	font-size: 7.47pt;
}
table table table {
	font-size: 6.5pt;
}
table table table table {
	font-size: 5.65pt;
}
table th {
	background-color:#e0e0e0;
}
table table th {
	background-color:#e8e8e8;
}
table table table th {
	background-color:#f0f0f0;
}
table table table table th {
	background-color:#f8f8f8;
}
.htd {
	text-align: right;
	width: 1px;
	white-space: nowrap;
	font-weight: bold;
}
.nb {
	break-inside: avoid;
}
.cen {
	text-align: center;
}
.r {
	text-align: right;
}
.rbt {
	text-align: right;
	border-top: 1px solid black;
}
th {
	padding: 1px;
	vertical-align: center;
	text-align: center;
	border-right: 2px solid white;
}
th:last-child {
	border-right: 0px solid white;
}
td {
	vertical-align: top;
	padding-right: 2px;
	padding-top: 1px;
	padding-bottom: 1px;
}
tr:last-child td {
	padding-bottom: none;
}
td:last-child {
	padding-right: none;
}
tr {
	break-inside: avoid;
}
.tot {
	margin-left: 50%;
	width: 50%;
}
table.tot {
	margin-top: 1px;
	break-inside: avoid;
}
table.tot tr:first-child {
	border-top: 1px solid;
}
hr {
	border: none;
	height: 1px;
	background-color: black;
	break-after: avoid;
}
ul {
	margin: 0;
}