TRANSFORMS = {
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas}AuxiliarCtas': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/AuxiliarCtas/AuxiliarCtas_{v}.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarCtas}AuxiliarCtas': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/AuxiliarCtas/AuxiliarCtas_{v}.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios}RepAuxFol': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/AuxiliarFolios/AuxiliarFolios_{v}.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/AuxiliarFolios}RepAuxFol': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/AuxiliarFolios/AuxiliarFolios_{v}.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion}Balanza': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/BalanzaComprobacion/BalanzaComprobacion_{v}.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/BalanzaComprobacion}Balanza': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/BalanzaComprobacion/BalanzaComprobacion_{v}.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas}Catalogo': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/CatalogoCuentas/CatalogoCuentas_{v}.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/CatalogoCuentas}Catalogo': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/CatalogoCuentas/CatalogoCuentas_{v}.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo}Polizas': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/PolizasPeriodo/PolizasPeriodo_{v}.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/PolizasPeriodo}Polizas': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/PolizasPeriodo/PolizasPeriodo_{v}.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_1/SelloDigitalContElec}SelloDigitalContElec': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/SelloDigitalContElec/SelloDigitalContElec.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_1/SelloDigitalContElec}SelloDigitalContElec': 'www.sat.gob.mx/esquemas/ContabilidadE/{v}/SelloDigitalContElec/SelloDigitalContElec.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas}AuxiliarCtas': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarCtas/AuxiliarCtas_1_2.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios}RepAuxFol': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/AuxiliarFolios/AuxiliarFolios_1_2.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion}Balanza': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/BalanzaComprobacion/BalanzaComprobacion_1_2.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas}Catalogo': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/CatalogoCuentas/CatalogoCuentas_1_2.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo}Polizas': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/PolizasPeriodo/PolizasPeriodo_1_2.xslt',
    '{www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec}SelloDigitalContElec': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec/SelloDigitalContElec.xslt',
    '{http://www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec}SelloDigitalContElec': 'www.sat.gob.mx/esquemas/ContabilidadE/1_3/SelloDigitalContElec/SelloDigitalContElec.xslt',
    '{http://www.sat.gob.mx/esquemas/retencionpago/1}Retenciones': 'www.sat.gob.mx/esquemas/retencionpago/1/retenciones.xslt',
    '{http://www.sat.gob.mx/esquemas/retencionpago/2}Retenciones': 'www.sat.gob.mx/esquemas/retencionpago/2/retenciones.xslt',
    '{http://www.sat.gob.mx/cfd/3}Comprobante': 'www.sat.gob.mx/sitio_internet/cfd/3/cadenaoriginal_{v}/cadenaoriginal_{v}.xslt',
    '{http://www.sat.gob.mx/cfd/4}Comprobante': 'www.sat.gob.mx/sitio_internet/cfd/4/cadenaoriginal_{v}/cadenaoriginal_{v}.xslt',
    '{http://www.sat.gob.mx/TimbreFiscalDigital}TimbreFiscalDigital': 'www.sat.gob.mx/sitio_internet/cfd/TimbreFiscalDigital/cadenaoriginal_TFD_{v}.xslt',
    '{http://www.sat.gob.mx/vehiculousado}VehiculoUsado': 'www.sat.gob.mx/sitio_internet/cfd/vehiculousado/vehiculousado.xslt',
}
